<?php
/*
 * Plugin Name: Flooows Elementor PRO Lead DB
 * Plugin URI:  https://www.flooows.com/
 * Description: An easy-to-use plugin to store and export Elementor PRO form leads.
 * Author:      Marco Ciambricco
 * Author URI:  https://www.fuerteweb.es
 * Version:     1.2.1
 */
 
 
define( 'ffls_path', plugin_dir_path( __FILE__ ) );

add_action( 'admin_menu', 'ffls_plugin_menu' );
add_action( 'plugins_loaded', 'ffls_contact_store_main_function' );
add_action( 'admin_action_ffls_export' , 'ffls_export_admin' );
	
 
	function ffls_export_admin()
	{	
		if(!empty($_POST['ffls-form-id']) and !empty($_POST['ffls-form-slug'])){
			
			$forms = ffls_get_published_elementor_forms();
			$formId =  sanitize_key($_POST['ffls-form-id']);
			$formSlug =  sanitize_key($_POST['ffls-form-slug']);
			/* Change var type to array */
			$post_ids = array($formId);
			$form_slugs = array($formSlug);
			
			$args = array(
					'post_type' => array( 'els_lead' ),
					'posts_per_page' => '-1',
					'meta_query' => array(
						'relation' => 'OR',
						array(
							'key'     => 'form_slug',
							'value'   => $form_slugs,
							'compare' => 'IN',
						),
						array(
							'key'     => 'post_id',
							'value'   => $post_ids,
							'compare' => 'IN',
						)
					)
				);			
					
			$query = new WP_Query( $args );
			$leads = $query->posts;
			wp_reset_postdata();
			
			foreach( $forms as $form ){
				if( $form['form_data']->id == $formSlug ){					
					$columns = array();
					foreach($form['form_data']->settings->form_fields as $field){
						$field_label = $field->field_label;
						$field_id = isset($field->custom_id) && $field->custom_id ? $field->custom_id : $field->_id;
						$columns[$field_id] = $field_label;
					}
					$colOrder = array();
					foreach( $columns as $key=>$value ){											
						$colOrder[] = $value;
					}	
				}
			}
			
			if( count($leads) ){
				$output = fopen("php://output",'w') or die("Can't open php://output");
				header("Content-Type:application/csv"); 
				header("Content-Disposition:attachment;filename=form_".$formSlug."_leads_export.csv");
				
				/* Add the Head Row */
				$leadFieldsHeadRow = array();
				/*First Column is the Lead Generation Date*/
				$leadFieldsHeadRow[] = 'Lead Generation Date';
				foreach( $colOrder as $column ){
					$leadFieldsHeadRow[] = $column;								
				}
				$leadFieldsHeadRow[] = 'IP Address';
				fputcsv($output, $leadFieldsHeadRow);
				/* Head Row END */
				foreach( $leads as $lead ){		
					$lead_fields = get_post_meta($lead->ID,'lead_data',true);
					$lead_fields = $lead_fields ? json_decode($lead_fields,true) : false;
					$lead_fields_row = array();
					/* Add the Lead Generation Date as first value of the row */
					$lead_fields_row[] = get_the_date( 'F j, Y' , $lead->ID);
					foreach($lead_fields as $key => $value){						
						foreach( $colOrder as $col ){
							if( $col == $value['title'] ) $lead_fields_row[] = $value['value'];
						}	
					}
					/* Add the IP address as last value of the row */					
					$lead_fields_row[] = get_post_meta($lead->ID,'lead_IP',true);
					
					fputcsv($output, $lead_fields_row);	
				}				
				fclose($output) or die("Can't close php://output");				
			}
		}		
	}
    function ffls_contact_store_main_function(){  
		add_action( 'admin_enqueue_scripts', 'ffls_load_css' );
		add_action( 'admin_enqueue_scripts', 'ffls_enqueue_js' );
		add_action( 'elementor_pro/forms/new_record', 'ffls_lead_store' ,  10, 10);		
    }
	
	function ffls_load_css() {
		$plugin_url = plugin_dir_url( __FILE__ );		
		wp_enqueue_style( 'ffls_css', $plugin_url . 'assets/css/ffls.css' );
	}
	function ffls_enqueue_js(){
		wp_enqueue_script('ffls_js', plugins_url( '/assets/js/ffls-script.js', __FILE__ ));
	}
	function ffls_plugin_menu() {
		wp_register_style( 'ffls', plugins_url( '/assets/css/ffls.css', __FILE__ ) );
		add_menu_page( 'Flooows Form Leads Store', 'Flooows Form Leads Store', 'manage_options', 'elementor-leads-store', 'ffls_plugin_options' ,'dashicons-id-alt');
	}
	function ffls_plugin_options() {
		if ( !current_user_can( 'manage_options' ) )  {
			wp_die(  'You do not have sufficient permissions to access this page.'  );
		}
		echo '<div><h1 class="ffls-heading-inline"><span class="dashicons dashicons-id-alt"></span>Flooows Form Leads Store for Elementor PRO</h1></div>';
		echo '<div><p> Choose the form you want to see the leads</p></div>';
		$forms = ffls_get_published_elementor_forms();
		if ( is_array($forms) ) {
			if( !count($forms) ){
				echo '<div><p> No Forms founded. You need to creat first an Elementor form!</p></div>' ;
			}else{
				echo  '<table class="widefat">
							<thead>
							<tr>							
								<th><strong>Form Name</strong></th>	
								<th><strong>Form URL</strong></th>							
								<th><strong>Leads Stored</strong></th>
								<th>Export to CSV</th>
							</tr>
							</thead>
							
							<tbody>';
				
				foreach( $forms as $form ){
					$formId = isset($form['post_id']) ? $form['post_id'] : 0;
					$formSlug = $form['form_data']->id;							
					/* Change var type to array */
					$post_ids = array($formId);
					$form_slugs = array($formSlug);
					
					if($included_posts = get_post_meta($formId,'_elementor_global_widget_included_posts',true)){
						$post_ids = array_keys($included_posts);
						foreach($post_ids as $included_post_id ){
							$elementor_data = get_post_meta($included_post_id,'_elementor_data',true);
							$elementor_data_json = json_decode($elementor_data,true);

							$slug = recursive_get_forms_slugs($elementor_data_json,$formId);
							if($slug){
								$form_slugs = array_merge($form_slugs,$slug);
							}
						}
						$post_ids[] = intval($formId);
					}
				
					$args = array(
						'post_type'              => array( 'els_lead' ),
						'posts_per_page'         => '-1',
						'meta_query' => array(
							'relation' => 'OR',
							array(
								'key'     => 'form_slug',
								'value'   => $form_slugs,
								'compare' => 'IN',
							),
							array(
								'key'     => 'post_id',
								'value'   => $post_ids,
								'compare' => 'IN',
							)
						)
					);
					
						
					$query = new WP_Query( $args );
					$count_leads = $query->post_count;
					wp_reset_postdata();	
					
					echo '<tr>';
					echo '<form method="POST" action="'. admin_url( 'admin.php' ).'">';				
					echo '<td>';
						echo '<a href="javascript:void(0);" onclick="showLeads(\''.esc_html__( $formSlug , 'text_domain' ).'\');">'.esc_html__( $form['form_data']->settings->form_name , 'text_domain' ).'</a>';
					echo '</td>';
					echo '<td>';
						$isGlobal = get_post_meta($formId,'_elementor_template_widget_type',true);
						$isGlobal = $isGlobal && 'form' === $isGlobal;
						$post_id_location = $isGlobal && $post_ids ? $post_ids[0] : $formId;
						echo "<a target='_blank' href='".get_permalink($post_id_location)."'>".get_the_title($post_id_location).($isGlobal ? ' ('.get_the_title($formId).')' : false)."</a></td>";					
					echo '</td>';
					echo '<td>';
						echo($count_leads);
					echo ' </td>';
					echo '<td>';
						echo '<input type="submit" title="Export leads in a CSV file" class="button ecs-button" name="export" value="Export">';
						echo '<input type="hidden" name="ffls-form-id" value="'.esc_html__( $formId , 'text_domain' ).'" />';
						echo '<input type="hidden" name="ffls-form-slug" value="'.esc_html__( $formSlug , 'text_domain' ).'" />';
						echo '<input type="hidden" name="action" value="ffls_export" />';
					echo ' </td>';
					echo '</form>';	
					echo '</tr> ';
					
					
				}
				
				echo '</tbody>
					</table>';
				
				/* For each form create the hidden DIV for the leads */
				foreach( $forms as $form ){
					
					$formId = isset($form['post_id']) ? $form['post_id'] : 0;
					$formSlug = $form['form_data']->id;
					
					$post_ids = array($formId);
					$form_slugs = array($formSlug);
					
					if($included_posts = get_post_meta($formId,'_elementor_global_widget_included_posts',true)){
						$post_ids = array_keys($included_posts);
						foreach($post_ids as $included_post_id ){
							$elementor_data = get_post_meta($included_post_id,'_elementor_data',true);
							$elementor_data_json = json_decode($elementor_data,true);

							$slug = recursive_get_forms_slugs($elementor_data_json,$formId);
							if($slug){
								$form_slugs = array_merge($form_slugs,$slug);
							}
						}
						$post_ids[] = intval($formId);
					}
				
					$args = array(
						'post_type'              => array( 'els_lead' ),
						'posts_per_page'         => '-1',
						'meta_query' => array(
							'relation' => 'OR',
							array(
								'key'     => 'form_slug',
								'value'   => $form_slugs,
								'compare' => 'IN',
							),
							array(
								'key'     => 'post_id',
								'value'   => $post_ids,
								'compare' => 'IN',
							)
						)
					);
					
					$query = new WP_Query( $args );
					$leads = $query->posts;
					wp_reset_postdata();				
							
					if( !count($leads) ){
						echo '<div class="ffls-hidden-leads-table" id="ffls-leads-table-'.esc_html__( $formSlug , 'text_domain' ).'"><h1 class="ffls-heading-inline" >'.esc_html__( $form['form_data']->settings->form_name , 'text_domain' ).'</h1>';
						echo '<p> There are no Leads generated by this form!</p>';
						echo '</div>';
					}else{
						echo '<div class="ffls-hidden-leads-table" id="ffls-leads-table-'.esc_html__( $formSlug , 'text_domain' ).'"><h1 class="ffls-heading-inline">'.esc_html__( $form['form_data']->settings->form_name , 'text_domain' ).'</h1>';
						$columns = array();
						foreach($form['form_data']->settings->form_fields as $field){
							$field_label = $field->field_label;
							
							$field_id = isset($field->custom_id) && $field->custom_id ? $field->custom_id : $field->_id;
							$columns[$field_id] = $field_label;
						}
						
						echo '<table class="widefat">
							<thead>
							<tr>';
						echo '<th><strong>Lead Generation Date</strong></th>';	
						$colOrder = array();
						foreach( $columns as $key=>$value ){
							if($value=='') echo '<th><strong>'.esc_html__( '-' , 'text_domain' ).'</strong></th>';
							else echo '<th><strong>'.esc_html__( $value , 'text_domain' ).'</strong></th>';	
							$colOrder[] = $value;
						}		
						echo '<th></th>
							</tr>
							</thead>						 
							<tbody>';
						
						foreach( $leads as $lead ){						
							echo '<tr>';
							echo '<td>';	
								echo get_the_date( 'F j, Y' , $lead->ID);
							echo '</td>';
							$lead_fields = get_post_meta($lead->ID,'lead_data',true);
							$lead_fields = $lead_fields ? json_decode($lead_fields,true) : false;
							foreach($lead_fields as $key => $value){
								foreach( $colOrder as $col ){
									if( $col == $value['title'] ) {
										$valueToShow = ffls_display_value($value);
										echo'<td>'.$valueToShow .'</td>';						
									}	
								}	
							}
							echo '</tr>';
						}
						echo '</tbody>
							</table>';
						echo '</div>';
					}
				}
			}	
		}else{
			echo '<div><p> No Forms founded. You need to creat first an Elementor form!</p></div>' ;
		}
	}
	
	
	function ffls_lead_store( $record ) {
		
		if( !isset($_POST['post_id']) || !isset($_POST['form_id']) ){
			return false;
		}
		
		$post_id = sanitize_key($_POST['post_id']);
		$form_slug = sanitize_key($_POST['form_id']);
		$form_name = $record->get_form_settings('form_name');
		$lead_data = $record->get('fields');	
		$usr_ip = sanitize_key($_SERVER['REMOTE_ADDR']);
		$usr_agent =  sanitize_key($_SERVER['HTTP_USER_AGENT']);			
		
		$meta['lead_data'] = wp_slash( wp_json_encode( $lead_data ) );
		$meta['post_id'] = $post_id;
		$meta['form_slug'] = $form_slug;
		$meta['lead_IP'] = $usr_ip;
		$meta['lead_user_agent'] = $usr_agent;
		
		$lead_id = wp_insert_post(
			array(
				'post_type'		=> 'els_lead',
				'post_status'	=> 'publish',
				'meta_input'	=> $meta,
			)
		);
		if($lead_id){			
			$post['ID'] = $lead_id;
			$post['post_title'] = 'Elementor PRO Lead #'.$lead_id;
			$post['post_name'] = 'elementor-leads-'.$lead_id;
			wp_update_post($post);
		}	
		
	}   
	function ffls_get_published_elementor_forms(){
		global $wpdb;
		$sql_query = "SELECT *  FROM `{$wpdb->prefix}postmeta`
		WHERE `meta_key` LIKE '_elementor_data'
		AND `meta_value` LIKE '%\"widgetType\":\"form\"%'
		AND `post_id` IN (
			SELECT `id` FROM `{$wpdb->prefix}posts`
			WHERE `post_status` IN ('publish','draft','private')
		)";

		$results = $wpdb->get_results($sql_query);
		if (!count($results)){
			return;
		}
		$forms = array();
		foreach($results as $result){
			$post_id = $result->post_id;
			$data = $result->meta_value;
			$json = json_decode($data);
			foreach( $json as $item ){				
				foreach( $item->elements as $element ){
					if ( $element->elType === 'widget' && $element->widgetType === 'form' ) {	
						$forms[] = array(
							'post_id' => $post_id,
							'form_data' => $item,
						);
					}
					
					if( !empty($element->elements) ){
						foreach( $element->elements as $el ){
							if ( $el->elType === 'widget' && $el->widgetType === 'form' ) {	
								$forms[] = array(
									'post_id' => $post_id,
									'form_data' => $el,
								);
							}					
						}
					}
				}
			}
			
		}	
		
		return $forms;
	}

	function ffls_get_leads( $enquires_id_array = null){
		
		$args = array(
		  'post_type'   => 'ffls_lead',
		); 
		$lastest_leads = get_posts( $args );		
		
		return $lastest_leads;
	}
	function ffls_display_value($value){

		$field_value = $value['value'];
		$valueToShow = '';
		if(!$field_value){
			$valueToShow = "-";
			return $valueToShow;
		}
		switch($value['type']){
			case "email":
				$valueToShow = "<a dir='ltr' target='_blank' href='mailto:$field_value'>$field_value</a>";
				break;
			case "tel":
				$valueToShow = "<a dir='ltr' href='tel:$field_value'>$field_value</a>";
				break;
			case "textarea":
				$valueToShow = nl2br($value['value']);
				break;
			case "url":
				$valueToShow = "<a target='_blank' href='$field_value'>".urldecode($field_value)."</a>";
				break;
			case "acceptance":
				$valueToShow = '<input type="checkbox" checked disabled style="opacity:1">';
				break;
			case "checkbox":
				$checks = explode(',',$value['value']);
				$count = 0;
				$valueToShow = '';
				$count_checks = count($checks);
				foreach($checks as $val){
					$count++;
					$valueToShow .= "$val";
					if($count_checks != $count){
						$valueToShow .= ",";
					}
				}
				break;
			case "upload":
				$valueToShow = "<a target='_blank' href='$field_value'>Download File</a>"; 
				break;
			default:
				$valueToShow = $field_value;
		}
		return $valueToShow;

	}
	/* debug function */
	function ffls_var_error_log( $object=null ){
		ob_start();                    // start buffer capture
		var_dump( $object );           // dump the values
		$contents = ob_get_contents(); // put the buffer into a variable
		ob_end_clean();                // end capture
		error_log( $contents );        // log contents of the result of var_dump( $object )
	}
  ?>